import { ResourceInfo } from "@/define/resource";
import { ActionWithType } from "../../Engine/CatanEngine/CSharp/System/ActionWithType";
import IResourceItem from "./IResourceItem";
import CardCouponItem from "./Items/CardCouponItem";
import LpPointItem from "./Items/LpPointItem";
import MoneyItem from "./Items/MoneyItem";
import { ResourceItemType } from "./ResourceItemType";


class UpdateResourceEvent extends ActionWithType<ResourceItemType, any[]> {
}

export default class CSResource {
	private static _event: UpdateResourceEvent;

	/** 更新資源事件 */
	public static get Event() {
		return CSResource._event;
	}

	public static Initialize(): void {
		CSResource._event = new UpdateResourceEvent();
		// Cocos.VicKing_Bridge.GetCSResourceEvent = () => { return CSResource._event; };
		// Cocos.VicKing_Bridge.AddResourceItemFromServer = (data: JSON) => { return CSResource.AddResourceItemFromServer(data); };
	}

	/**
	 * 新增一筆資源(資料必須是從Server來的)
	 * @param data ex [1, 15] --> 新增元寶資源
	 */
	public static AddResourceItemFromServer(data: ResourceInfo | JSON): IResourceItem {
		let item = this.GetResourceItemFromServer(data);
		item.AddToResource();
		return item;
	}

	/**
	 * 新增多筆資源(資料必須是從Server來的)
	 * @param data ex: [[1, 15], [2, 30], [28, ["0000123", [124, 3, 0, 0, 0, 0, 0, 0, 0, 1000, 0, 0, 1, 1, 0.000, 0.00, 1, 0, 0, 0, 0, 0, 3]]]] --> 新增[元寶, 銀兩, 卡片]等資源
	 */
	public static AddResourceItemsFromServer(data: ResourceInfo[] | JSON): IResourceItem[] {
		let itemList: IResourceItem[] = [];
		for (let i = 0, count = Object.keys(data).length; i < count; i++) {
			let itemData: JSON = data[i];
			itemList.push(this.AddResourceItemFromServer(itemData));
		}
		return itemList;
	}

	/**
	 * 從Server資料還原對應的資源物件
	 * @param data ex: [1, 15] --> 元寶, [2, 30] --> 銀兩
	 */
	public static GetResourceItemFromServer(data: ResourceInfo | JSON): IResourceItem {
		let type = data[0];
		switch (type) {
			case ResourceItemType.Money:
				return new MoneyItem(data);
			case ResourceItemType.Card_Coupon:
				return new CardCouponItem(data);
			case ResourceItemType.LpPoint:
				return new LpPointItem(data);
			default:
				throw new Error("沒有對應[" + type + "]類型的Server資源類別");
		}
	}

	/**
	 * 從Server資料還原對應的資源物件
	 * @param data ex: [[1, 15], [2, 30], [45, [1, 10], ...] --> [元寶, 銀兩, 節慶貨幣, ...]
	 */
	public static GetResourceItemsFromServer(data: JSON): IResourceItem[] {
		let itemList: IResourceItem[] = [];
		for (let i = 0, count = Object.keys(data).length; i < count; i++) {
			let itemData: JSON = data[i];
			itemList.push(this.GetResourceItemFromServer(itemData));
		}
		return itemList;
	}

	/**
	 * 新增多筆資源(直接拿資料本身)
	 * @param items
	 */
	public static AddResourceItems(items: IResourceItem[]): void {
		for (let item of items) {
			item.AddToResource();
		}
	}

	// #endregion
}