import { ResourceInfo } from "@/define/resource";
import { NetRequest } from "@/Engine/CatanEngine/NetManagerV2/NetRequest";

// #region Request

export type RpcTaskInfoRequest = null
export type RpcTaskInfoResponse = TaskInfo[]
export class TaskInfoRequest extends NetRequest<RpcTaskInfoRequest, RpcTaskInfoResponse> {
	constructor() {
		super();
	}

	get Method(): string {
		return "task.info";
	}
}

export type RpcTaskLogRequest = null
export type RpcTaskLogResponse = TaskInfo[]
export class TaskLogRequest extends NetRequest<RpcTaskLogRequest, RpcTaskLogResponse> {
	constructor() {
		super();
	}

	get Method(): string {
		return "task.log";
	}
}

export type RpcTaskProcessRequest = {
	act: number;
	id: TaskAction;
};
export type RpcTaskProcessResponse = TaskInfo | null | ResourceInfo[]
export class TaskProcessRequest extends NetRequest<RpcTaskProcessRequest, RpcTaskProcessResponse> {
	constructor(act: TaskAction, id: number) {
		super();
		this.Data = {
			act,
			id
		};
	}

	get Method(): string {
		return "task.process";
	}
}

// #endregion

// #region Type

export type TaskInfo = [
	taskType: TaskType,
	taskID: number,
	taskProgress: number[],
	dailyTaskStatus: DailyTaskStatus[],
	receiveTaskTime: number,
	taskStatus: TaskStatus | undefined
];

// #endregion

// #region Enum

export enum TaskType {
	Normal = 1,
	Advance
}

export enum DailyTaskStatus {
	NotYetStarted,
	Success,
	Failure,
	InProgress
}

export enum TaskStatus {
	Completed = 1,
	Failure,
	GiveUp,
	InProgress
}
export enum TaskAction {
	TakeTask,
	GiveUp,
	Complete
}

// #endregion
